import requestAPI from "./xhrApi";

let isFormData = payload => {
  if (typeof FormData !== "undefined" && payload) {
    return payload.constructor == new FormData().constructor;
  }

  return typeof payload == "function";
};

let Builder = (apiJson, options) => {
  //Url and param builder..
  var apiOptions = function () {
    let _method = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    let defaultParameters = arguments.length > 1 ? arguments[1] : undefined;
    let allowedParameters = arguments.length > 2 ? arguments[2] : undefined;
    let requestUrl = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '';
    let parameters = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
    let queryParameters = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};
    var isFormDataFound = isFormData(parameters);
    var userParamsClone = Object.assign({}, isFormDataFound ? queryParameters : parameters); // to address re-back call ajax parameters keeping..

    if (Object.keys(userParamsClone).length) {
      var propsToUrl = requestUrl.split('/'); // helps to some parameter to url...

      for (var i = 0, len = propsToUrl.length; i < len; i++) {
        var prop = propsToUrl[i];
        var paramKey = prop.replace(':', '');

        if (prop && prop.charAt(0) === ':' && userParamsClone.hasOwnProperty(paramKey)) {
          var value = userParamsClone[paramKey];
          requestUrl = requestUrl.replace('/' + prop, value ? '/' + value : '');
          requestUrl = requestUrl.replace(prop + '/', value ? value + '/' : '');
          delete userParamsClone[paramKey];
        }
      }
    }

    let userParams = isFormDataFound ? parameters : userParamsClone;
    queryParameters = isFormDataFound ? userParamsClone : queryParameters;
    let defaultParams = Object.assign({}, queryParameters, options.defaultParams(_method));
    let gobalParamsToUrl = '';
    let count = 0;

    for (var prop in defaultParams) {
      if (count > 0) {
        gobalParamsToUrl += `&${prop}=${defaultParams[prop]}`;
        break;
      }

      gobalParamsToUrl += `${prop}=${defaultParams[prop]}`;
      count = 1;
    }

    gobalParamsToUrl = gobalParamsToUrl ? '?' + gobalParamsToUrl : '';

    if (!isFormDataFound) {
      var length = userParams ? Object.keys(userParams).length : 0;
      userParams = length > 0 ? userParams : null;
    }

    if (defaultParameters && userParams) {
      userParams = Object.assign({}, defaultParameters || {}, userParams);
    }

    if (allowedParameters && allowedParameters.length && userParams) {
      let newUserParams = {};
      allowedParameters.forEach(paramName => {
        if (userParams.hasOwnProperty(paramName)) {
          newUserParams[paramName] = userParams[paramName];
        }
      });
      userParams = newUserParams;
    }

    return {
      url: options.prefixUrl(_method) + requestUrl + gobalParamsToUrl,
      param: userParams
    };
  }; //url - bind arguments not from user..


  var httpConnection = (ajaxmethod, defaultParameters, allowedParameters) => {
    return function (url) {
      let ajaxoptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      let ajaxoption2 = arguments.length > 2 ? arguments[2] : undefined;
      let ajaxoption3 = arguments.length > 3 ? arguments[3] : undefined;
      let urlHeaders = typeof options.authHeaders === "function" ? options.authHeaders(ajaxmethod) : null;
      let customOptions = options.customOptions || {};

      if (typeof ajaxoptions === "function" || typeof ajaxoption2 === "function") {
        let success = typeof ajaxoptions === "function" ? ajaxoptions : ajaxoption2;
        let failure = typeof ajaxoptions === "function" ? ajaxoption2 : ajaxoption3;
        let payload = typeof ajaxoptions === "function" ? null : ajaxoptions;

        let successCallback = response => {
          customOptions.hideLoader && customOptions.hideLoader(ajaxmethod);
          return success ? success(response) : response;
        };

        let failureCallback = errResponse => {
          customOptions.hideLoader && customOptions.hideLoader(ajaxmethod);
          return failure ? failure(errResponse) : errResponse;
        };

        let apioptions = apiOptions(ajaxmethod, defaultParameters, allowedParameters, url, payload);
        customOptions.showLoader && customOptions.showLoader(ajaxmethod);
        let {
          param
        } = apioptions;

        if (param && param.headers) {
          urlHeaders = Object.assign(urlHeaders || {}, param.headers);
          delete param.headers;
        }

        return requestAPI(apioptions.url, urlHeaders)[ajaxmethod](apioptions.param).then(successCallback, failureCallback);
      }

      let {
        payload = {},
        headers,
        parameters,
        success,
        failure,
        progressReport,
        abortSignal
      } = ajaxoptions;
      urlHeaders = urlHeaders ? Object.assign(urlHeaders, headers) : headers; // Override headers

      let successCallback = response => {
        customOptions.hideLoader && customOptions.hideLoader(ajaxmethod);
        return success ? success(response) : response;
      };

      let failureCallback = errResponse => {
        customOptions.hideLoader && customOptions.hideLoader(ajaxmethod);
        return failure ? failure(errResponse) : errResponse;
      };

      let apioptions = apiOptions(ajaxmethod, defaultParameters, allowedParameters, url, payload, parameters);
      customOptions.showLoader && customOptions.showLoader(ajaxmethod);
      return requestAPI(apioptions.url, urlHeaders)[ajaxmethod](apioptions.param, progressReport, abortSignal).then(successCallback, failureCallback);
    };
  }; //auto api functions creation...


  var userAPIS = {};

  for (var prop in apiJson) {
    var api = prop;
    var apivalue = apiJson[prop];
    var apiurl = apivalue.url;
    var apimethod = apivalue.method;
    var defaultParameters = apivalue.defaultParameters;
    var allowedParameters = apivalue.allowedParameters; //var apiFragments = api.split('/');

    var apiFragments = api.split('.');
    var lastObj = userAPIS;

    for (var j = 0; j < apiFragments.length; j++) {
      var apiFragment = apiFragments[j];

      if (apiFragment.indexOf(':') != -1) {
        continue; //Skipping this !api
      }

      if (apiFragment && typeof lastObj[apiFragment] === 'undefined') {
        lastObj[apiFragment] = httpConnection(apimethod, defaultParameters, allowedParameters).bind(null, apiurl);
      }

      if (apiFragment && typeof lastObj[apiFragment] !== 'undefined') {
        lastObj = lastObj[apiFragment];
      }
    }
  }

  return userAPIS;
};

export default class {
  constructor(prefixUrl, defaultParams, authHeaders, options) {
    this.prefixUrl = prefixUrl;
    this.defaultParams = defaultParams;
    this.authHeaders = authHeaders;
    this.options = options;
  }

  create(javaApiList, name) {
    let defaultParams = this.defaultParams ? this.defaultParams.bind(null, name) : null;
    let prefixUrl = this.prefixUrl ? this.prefixUrl.bind(null, name) : '';
    let authHeaders = this.authHeaders ? this.authHeaders.bind(null, name) : null;
    let customOptions = this.options ? this.options : null;
    return Builder(javaApiList || {}, {
      defaultParams,
      prefixUrl,
      authHeaders,
      customOptions
    });
  }

}